# SyllogisTIC

Despite what the README says, this is a package resolver, preprocessor,
and builer all under one roof, for better or worse. (Probably worse.)

## Flags

### --strict

Turns warnings into errors.

### --out

Filename to output to instead of the default `[filename].build.[extensions]`

### --watch

Watches the input and all included files for changes.

## Feature Groups

Code can bundle blocks into a feature group, so long as the code is valid after
any code between the start and end is removed.

Groups can be nested. Outer group visibility will have priority over inner group
visibility. That means even if an inner group is enabled, it won't show up in
the build if an outer group is disabled.

**Group names should ideally be a single 'word' and cannot include a comma or
spaces.** + or - characters are acceptable as long as they're not the first
character in a group name, to avoid confusion with the enable/disable
indicators.

(A future version might allow quoting group names to allow spaces, but don't
bet on it.)

### Group Enabled by Default

```lua
-- #group:start sample
local code='goes here'
-- #group:end sample
```

The '+' is optional, but recommended just to be explicit.

```lua
-- #group:start +sample
local code='goes here'
-- #group:end sample
```

Group ends do not have any enable/disable prefix.

### Group Disabled by Default
```
-- #group:start -sample
local code='goes here'
-- #group:end sample
```

### Nested Groups

```
-- #group:start -sample
local code='goes here'
-- #group:end sample

-- #group:start +test
local hello='world'
-- #group:start -sample
local code='goes here'
-- #group:end sample
-- #group:end test
```

**Groups cannot be cross nested. Not OK:**

```lua
-- #group:start test
local hello='world'
-- #group:start -sample
local code='goes here'
-- #group:end test
local more='code'
-- #group:end sample
```

## Imports

**SyllogisTIC does not support imports** for a few important reasons:

- The author just doesn't want to do it
- It's complicated

If imports similar to JavaScript and Python were supported, existing code
parsers would help us find the items we needed to copy over. However, as the
TIC-80 doesn't natively support imports or the concept of modules, these copied
variables and functions would need to *also* copy over variables, functions, and
parent scopes they reference until there's no more dependencies to resolve.

That'd involve a lot of AST traversing and get really weird fast.

If you don't like it, feel free to write your own build system. 😁

## Includes

Includes copy the given file into the current file at the location specified by
the magic #include comment.

Includes must start with a '.' (relative) or a '/' (\*nix-style absolute). When
not referencing an index (see below) the full file name, including extension, is
required.

Includes that start with neither character will be resolved as a 'package'.
SyllogisTIC looks for packages in `.tic_pkgs` starting in the current directory
then going up directories until the package is found. If the root is reached
without finding the package, it looks in `~/.tic_pkgs`

Packages that include other packages will be looked up in the same manner,
**relative to the file being built**.

Nested includes are supported, but for the moment if two files ask to include
the same file, it will get included twice and emit a warning.

Future versions may have better behavior such that it only includes a given file
or package once per project, with a properly intersected include/exclude lists.

### Single File

```lua
-- #include '/path/to/file.lua'
```

```lua
-- #include './path/to/file.lua'
```

### Directory Roll-Up (indexes)

This assumes there is an 'index.lua' file inside the referenced directory.

```lua
-- #include './path/to/directory'
```

### Packages

```lua
-- #include 'freighty' +wu
```

```lua
-- #include 'freighty/core'
```

**Package names must start with a letter, number, or underscore.** Other start
characters are reserved for future updates (different resolution methods, etc.)

### Toggling Feature Groups

Explicitly enabling a default-enabled group or disabling a default-disabled group are just noops.

#### Enabling

```lua
-- #include './path/to/file.lua' +sample
```

```lua
-- #include './path/to/file.lua' +sample,test
```

#### Disabling

```lua
-- #include './path/to/file.lua' -sample
```

```lua
-- #include './path/to/file.lua' -sample,test
```

#### Both

Enabling and disabling can be listed in any order. Both can have one or more arguments.

```lua
-- #include './path/to/file.lua' +sample,test -extra
```

**Technically also allowed:**
```lua
-- #include 'path/to/file.lua' +sample +test -extra
```

**Specifying a group as enabled and disabled on include is a warning and
behavior may change.**

### Import Namespacing

Sometimes it's desirable to rename things so they don't clash with other
import items, like if someone writes a framework that uses the most generic
names possible for their stuff. Don't be that person. 😅

The 'as' keyword, combined with export comments, will rename all known exports
to start with the given name and include a simple namespace definition block.

**Namespaces must be a valid Lua variable name.**

🚨 **This can have unexpected consequences when local variables
shadow exported items being namespaced.**

```lua
-- #include 'path/to/library' as lib3
-- #include 'path/of/file.lua' as other +test
```

**Example Input and Ouptut**

index.lua (library)
```lua
-- #include shape.lua
-- #include math3d.lua
```

file.lua
```lua
filestuff=3

-- #group:start -test
function debug()
  trace('hello')
end
-- #group:end test
```

Output (based on example includes):
```lua
lib3={}
lib3.shape={
  -- ...
}
function math3d.translate(s,x,y,z)
  -- ...
end
other.filestuff=3

function other.debug()
  trace('hello')
end
```

## Exports

Currently SyllogisTIC does not automatically determine what's exported from a
file, though it may do so in the future.

As such, an export statement is required to indicate what variables are exported
from the file. Exported names should match top-level variables without '.'

Exports are currently only used for the 'as' variant of includes.

Anything not declared as an export will still be included, but just won't be
transformed by an 'as' statement.

**Examples**

```lua
test={}

-- #exports test
```

```lua
shape={}
test={}

-- #exports shape,test
```
